@@ -5,8 +5,9 @@ use Moo::Role;
 use Scalar::Util qw/blessed/;
 use Carp qw/croak/;
 use Business::CPI::Util::Types qw/Money to_Money/;
+use List::Util qw/sum/;
 
-our $VERSION = '0.914'; # VERSION
+our $VERSION = '0.915'; # VERSION
 
 has id => ( is => 'rw' );
 has gateway_id => ( is => 'rw' );
@@ -41,6 +42,14 @@ has discount => (
     default => sub { 0 },
 );
 
+has shipping => (
+    coerce => \&to_Money,
+    isa    => Money,
+    is     => 'rw',
+    default => sub { 0 },
+);
+
+
 has _gateway => (
     is       => 'ro',
     required => 1,
@@ -60,6 +69,42 @@ has _receivers => (
     default => sub { [] },
 );
 
+sub get_total_shipping {
+    my ($self) = @_;
+
+    my $amount = 0;
+
+    foreach my $item (@{ $self->_items }) {
+        my $item_shipping = 0;
+
+        if ($item->has_shipping) {
+            $item_shipping = $item->shipping +
+              ( $item->quantity - 1 ) *
+              (   $item->has_shipping_additional
+                ? $item->shipping_additional
+                : $item->shipping );
+        }
+
+        $amount += $item_shipping;
+    }
+
+    return $amount + $self->shipping;
+}
+
+sub get_total_amount {
+    my ($self) = @_;
+
+    my $amount = sum( map { $_->price * $_->quantity } @{ $self->_items } );
+
+    $amount +=
+      $self->get_total_shipping +
+      $self->tax +
+      $self->handling -
+      $self->discount;
+
+    return $amount;
+}
+
 sub get_item {
     my ($self, $item_id) = @_;
 
@@ -141,7 +186,7 @@ Business::CPI::Role::Cart - Shopping cart or an order
 
 =head1 VERSION
 
-version 0.914
+version 0.915
 
 =head1 DESCRIPTION
 
@@ -161,22 +206,37 @@ The id your gateway has set for this cart, if there is one.
 =head2 buyer
 
 The person paying for the shopping cart. See L<Business::CPI::Role::Buyer> or
-L<Business::CPI::Role::Account>.
+L<Business::CPI::Role::Account>. (This is a bit confusing because the interface
+isn't stable yet, and we are planning a transition from the Buyer role to the
+Account role. But most CPI gateways still use the Buyer role.)
 
 =head2 discount
 
-Discount to be applied to the total amount. Positive number.
+Discount to be subtracted from the total amount. Positive number.
 
 =head2 tax
 
-Tax to be applied to the total amount. Positive number.
+Tax to be added to the total amount. Positive number.
 
 =head2 handling
 
-Handling to be applied to the total amount. Positive number.
+Handling to be added to the total amount. Positive number.
+
+=head2 shipping
+
+Price of the shipping to be added to the total amount. Positive number.
 
 =head1 METHODS
 
+=head2 get_total_shipping
+
+Traverse all items from this cart and returns the sum of each shipping cost,
+plus the value of the shipping attribute.
+
+=head2 get_total_amount
+
+Calculates the total amount of the cart.
+
 =head2 add_item
 
 Create a new L<< Item | Business::CPI::Role::Item >> object with the given
